# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import os
import tempfile
import unittest
from pathlib import Path
from tempfile import TemporaryDirectory
from unittest.mock import Mock

from autohooks.hooks import PreCommitHook, get_pre_commit_hook_path
from autohooks.settings import Mode
from autohooks.template import (
    PIPENV_MULTILINE_SHEBANG,
    PIPENV_SHEBANG,
    POETRY_MULTILINE_SHEBANG,
    POETRY_SHEBANG,
    PYTHON3_SHEBANG,
    TEMPLATE_VERSION,
    PreCommitTemplate,
)
from autohooks.utils import exec_git


class GitDirTestCase(unittest.TestCase):
    def setUp(self):
        self.tempdir = TemporaryDirectory()
        self.temp_dir_path = Path(self.tempdir.name).resolve()

        exec_git("-C", str(self.temp_dir_path), "init")

        self.git_dir_path = self.temp_dir_path / ".git"

        self.assertTrue(self.git_dir_path.exists())

        os.chdir(str(self.temp_dir_path))

    def tearDown(self):
        self.tempdir.cleanup()


class GetPreCommitHookPathTestCase(GitDirTestCase):
    def test_get_path(self):
        pre_commmit_hook_path = get_pre_commit_hook_path()

        self.assertEqual(
            pre_commmit_hook_path,
            self.temp_dir_path / ".git" / "hooks" / "pre-commit",
        )


class InstallPreCommitHook(GitDirTestCase):
    def test_install(self):
        pre_commit_hook = PreCommitHook()

        self.assertFalse(pre_commit_hook.exists())

        pre_commit_hook.write(mode=Mode.PIPENV)

        self.assertTrue(pre_commit_hook.exists())


class FakeReadPath:
    def __init__(self, text):
        self._text = text

    def read_text(self):
        return self._text


class IsAutohooksPreCommitHook(unittest.TestCase):
    def test_other_hook(self):
        path = FakeReadPath("foo\nbar")
        pre_commit_hook = PreCommitHook(path)

        self.assertFalse(pre_commit_hook.is_autohooks_pre_commit_hook())

    def test_pre_commit_template(self):
        template = PreCommitTemplate()
        path = FakeReadPath(template.render(mode=Mode.PIPENV))
        pre_commit_hook = PreCommitHook(path)

        self.assertTrue(pre_commit_hook.is_autohooks_pre_commit_hook())


class IsCurrentAutohooksPreCommitHook(unittest.TestCase):
    def test_other_hook(self):
        path = FakeReadPath("foo\nbar")
        pre_commit_hook = PreCommitHook(path)

        self.assertFalse(pre_commit_hook.is_current_autohooks_pre_commit_hook())

    def test_pre_commit_template(self):
        template = PreCommitTemplate()
        path = FakeReadPath(template.render(mode=Mode.PIPENV))
        pre_commit_hook = PreCommitHook(path)

        self.assertTrue(pre_commit_hook.is_current_autohooks_pre_commit_hook())

    def test_modified_pre_commit_template(self):
        template = PreCommitTemplate()
        rendered = template.render(mode=Mode.PIPENV)
        lines = rendered.split("\n")
        lines[1] = ""
        path = FakeReadPath("\n".join(lines))
        pre_commit_hook = PreCommitHook(path)

        self.assertFalse(pre_commit_hook.is_current_autohooks_pre_commit_hook())


class ReadVersionTestCase(unittest.TestCase):
    def test_read_version(self):
        template = PreCommitTemplate()
        with tempfile.TemporaryDirectory() as tempdir:
            tmp_hook_path = Path(tempdir) / "pre-commit-test"
            # Find version using all shebang modes
            for mode in [m for m in Mode if m.value > 0]:
                with open(str(tmp_hook_path), "w", encoding="utf-8") as tmpfile:
                    tmpfile.write(template.render(mode=mode))
                pre_commit_hook = PreCommitHook(tmp_hook_path)

            self.assertEqual(TEMPLATE_VERSION, pre_commit_hook.read_version())

    def test_empty_content(self):
        path = FakeReadPath("")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_version(), -1)

    def test_no_meta(self):
        path = FakeReadPath("\n# foo = bar")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_version(), -1)


class ReadModeTestCase(unittest.TestCase):
    def test_undefined_mode(self):
        path = FakeReadPath("")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.UNDEFINED)

    def test_unknown_mode(self):
        path = FakeReadPath("#!foo")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.UNKNOWN)

    def test_pipenv_mode(self):
        path = FakeReadPath(f"#!{PIPENV_SHEBANG}")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.PIPENV)

    def test_poetry_mode(self):
        path = FakeReadPath(f"#!{POETRY_SHEBANG}")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.POETRY)

    def test_poetry_mode_with_python3(self):
        path = FakeReadPath(f"#!{POETRY_SHEBANG}3")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.POETRY)

    def test_pipenv_multiline_mode(self):
        path = FakeReadPath(f"#!{PIPENV_MULTILINE_SHEBANG}")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.PIPENV_MULTILINE)

    def test_poetry_multiline_mode(self):
        path = FakeReadPath(f"#!{POETRY_MULTILINE_SHEBANG}")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.POETRY_MULTILINE)

    def test_pythonpath_mode(self):
        path = FakeReadPath(f"#!{PYTHON3_SHEBANG}")
        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(pre_commit_hook.read_mode(), Mode.PYTHONPATH)


class WriteTestCase(unittest.TestCase):
    def test_pipenv_mode(self):
        write_path = Mock()
        pre_commit_hook = PreCommitHook(write_path)
        pre_commit_hook.write(mode=Mode.PIPENV)

        write_path.chmod.assert_called_with(0o775)
        self.assertTrue(write_path.write_text.called)

        args, _kwargs = write_path.write_text.call_args
        text = args[0]
        self.assertRegex(text, f"^#!{PIPENV_SHEBANG} *")

    def test_poetry_mode(self):
        write_path = Mock()
        pre_commit_hook = PreCommitHook(write_path)
        pre_commit_hook.write(mode=Mode.POETRY)

        write_path.chmod.assert_called_with(0o775)
        self.assertTrue(write_path.write_text.called)

        args, _kwargs = write_path.write_text.call_args
        text = args[0]
        self.assertRegex(text, f"^#!{POETRY_SHEBANG} *")

    def test_pythonpath_mode(self):
        write_path = Mock()
        pre_commit_hook = PreCommitHook(write_path)
        pre_commit_hook.write(mode=Mode.PYTHONPATH)

        write_path.chmod.assert_called_with(0o775)
        self.assertTrue(write_path.write_text.called)

        args, _kwargs = write_path.write_text.call_args
        text = args[0]
        self.assertRegex(text, f"^#!{PYTHON3_SHEBANG} *")


class StrTestCase(unittest.TestCase):
    def test_str_conversion(self):
        path = Mock()
        path.__str__ = Mock(return_value="foo")

        pre_commit_hook = PreCommitHook(path)

        self.assertEqual(str(pre_commit_hook), "foo")
        path.__str__.assert_called_with()


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2022-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import os
import unittest
from pathlib import Path

from autohooks.api.git import (
    Status,
    StatusEntry,
    get_staged_status,
    get_status,
    is_partially_staged_status,
    is_staged_status,
)
from tests import tempgitdir

from . import GitTestCase, git_add, git_commit, git_mv, git_rm


def init_test_repo(tmpdir: Path):
    tracked_file = tmpdir / "foo.json"
    tracked_file.write_text("sed diam nonumy eirmod", encoding="utf8")
    changed_file = tmpdir / "bar.json"
    changed_file.touch()
    staged_changed_file = tmpdir / "ipsum.json"
    staged_changed_file.write_text("tempor invidunt ut labore")
    removed_file = tmpdir / "lorem.json"
    removed_file.write_text("consetetur sadipscing elitr", encoding="utf8")
    renamed_file = tmpdir / "foo.md"
    renamed_file.write_text("et dolore magna aliquyam erat", encoding="utf8")

    git_add(
        tracked_file,
        changed_file,
        staged_changed_file,
        removed_file,
        renamed_file,
    )
    git_commit()

    changed_file.write_text("Lorem Ipsum", encoding="utf8")
    staged_changed_file.write_text("Lorem Ipsum", encoding="utf8")

    added_file = tmpdir / "foo.txt"
    added_file.touch()

    added_modified_file = tmpdir / "ipsum.txt"
    added_modified_file.touch()

    git_add(added_file, staged_changed_file, added_modified_file)

    staged_changed_file.write_text("Dolor sit", encoding="utf8")

    added_modified_file.write_text("Lorem Ipsum", encoding="utf8")

    git_mv(renamed_file, tmpdir / "foo.rst")

    git_rm(removed_file)

    untracked_file = tmpdir / "bar.txt"
    untracked_file.touch()

    return (
        tracked_file,
        changed_file,
        added_file,
        staged_changed_file,
        added_modified_file,
        removed_file,
        renamed_file,
        untracked_file,
    )


class StatusEntryTestCase(unittest.TestCase):
    def test_parse_modified_modified(self):
        status = StatusEntry("MM foo.txt")

        self.assertEqual(status.index, Status.MODIFIED)
        self.assertEqual(status.working_tree, Status.MODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_modified_unmodified(self):
        status = StatusEntry("M  foo.txt")

        self.assertEqual(status.index, Status.MODIFIED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_deleted(self):
        status = StatusEntry("D  foo.txt")

        self.assertEqual(status.index, Status.DELETED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_added(self):
        status = StatusEntry("A  foo.txt")

        self.assertEqual(status.index, Status.ADDED)
        self.assertEqual(status.working_tree, Status.UNMODIFIED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_untracked(self):
        status = StatusEntry("?? foo.txt")

        self.assertEqual(status.index, Status.UNTRACKED)
        self.assertEqual(status.working_tree, Status.UNTRACKED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_parse_ignored_untracked(self):
        status = StatusEntry("!? foo.txt")

        self.assertEqual(status.index, Status.IGNORED)
        self.assertEqual(status.working_tree, Status.UNTRACKED)
        self.assertEqual(status.path, Path("foo.txt"))

    def test_pathlike(self):
        status = StatusEntry("MM foo.txt")
        self.assertEqual(os.fspath(status), "foo.txt")


class GetStatusTestCase(GitTestCase):
    def test_get_status(self):
        with tempgitdir() as tmpdir:
            (
                _tracked_file,
                changed_file,
                added_file,
                staged_changed_file,
                added_modifed_file,
                removed_file,
                renamed_file,
                _untracked_file,
            ) = init_test_repo(tmpdir)

            status = get_status()
            self.assertEqual(len(status), 6)

            changed_file_status = status[0]
            renamed_file_status = status[1]
            added_file_status = status[2]
            staged_changed_file_status = status[3]
            added_modifed_file_status = status[4]
            removed_file_status = status[5]

            self.assertEqual(changed_file_status.absolute_path(), changed_file)
            self.assertEqual(changed_file_status.index, Status.UNMODIFIED)
            self.assertEqual(changed_file_status.working_tree, Status.MODIFIED)

            self.assertEqual(
                renamed_file_status.old_path.absolute(), renamed_file
            )
            self.assertEqual(renamed_file_status.index, Status.RENAMED)
            self.assertEqual(
                renamed_file_status.working_tree, Status.UNMODIFIED
            )

            self.assertEqual(added_file_status.absolute_path(), added_file)
            self.assertEqual(added_file_status.index, Status.ADDED)
            self.assertEqual(added_file_status.working_tree, Status.UNMODIFIED)

            self.assertEqual(
                staged_changed_file_status.absolute_path(), staged_changed_file
            )
            self.assertEqual(staged_changed_file_status.index, Status.MODIFIED)
            self.assertEqual(
                staged_changed_file_status.working_tree, Status.MODIFIED
            )

            self.assertEqual(
                added_modifed_file_status.absolute_path(), added_modifed_file
            )
            self.assertEqual(added_modifed_file_status.index, Status.ADDED)
            self.assertEqual(
                added_modifed_file_status.working_tree, Status.MODIFIED
            )

            self.assertEqual(removed_file_status.absolute_path(), removed_file)
            self.assertEqual(removed_file_status.index, Status.DELETED)
            self.assertEqual(
                removed_file_status.working_tree, Status.UNMODIFIED
            )

    def test_get_status_for_files(self):
        with tempgitdir() as tmpdir:
            (
                _tracked_file,
                changed_file,
                _added_file,
                _staged_changed_file,
                _added_modifed_file,
                _removed_file,
                renamed_file,
                _untracked_file,
            ) = init_test_repo(tmpdir)

            status = get_status((changed_file, renamed_file))

            self.assertEqual(len(status), 2)

            changed_file_status = status[0]
            renamed_file_status = status[1]

            self.assertEqual(changed_file_status.absolute_path(), changed_file)
            self.assertEqual(changed_file_status.index, Status.UNMODIFIED)
            self.assertEqual(changed_file_status.working_tree, Status.MODIFIED)

            # the status is deleted for the renamed file because the new file
            # path is not passed to get_status
            self.assertEqual(renamed_file_status.absolute_path(), renamed_file)
            self.assertEqual(renamed_file_status.index, Status.DELETED)
            self.assertEqual(
                renamed_file_status.working_tree, Status.UNMODIFIED
            )


class GetStagedStatusTestCase(GitTestCase):
    def test_get_staged_status(self):
        with tempgitdir() as tmpdir:
            (
                _tracked_file,
                _changed_file,
                added_file,
                staged_changed_file,
                added_modifed_file,
                _removed_file,
                renamed_file,
                _untracked_file,
            ) = init_test_repo(tmpdir)

            status = get_staged_status()
            self.assertEqual(len(status), 4)

            renamed_file_status = status[0]
            added_file_status = status[1]
            staged_changed_file_status = status[2]
            added_modifed_file_status = status[3]

            self.assertEqual(
                renamed_file_status.old_path.absolute(), renamed_file
            )
            self.assertEqual(renamed_file_status.index, Status.RENAMED)
            self.assertEqual(
                renamed_file_status.working_tree, Status.UNMODIFIED
            )

            self.assertEqual(added_file_status.absolute_path(), added_file)
            self.assertEqual(added_file_status.index, Status.ADDED)
            self.assertEqual(added_file_status.working_tree, Status.UNMODIFIED)

            self.assertEqual(
                staged_changed_file_status.absolute_path(), staged_changed_file
            )
            self.assertEqual(staged_changed_file_status.index, Status.MODIFIED)
            self.assertEqual(
                staged_changed_file_status.working_tree, Status.MODIFIED
            )

            self.assertEqual(
                added_modifed_file_status.absolute_path(), added_modifed_file
            )
            self.assertEqual(added_modifed_file_status.index, Status.ADDED)
            self.assertEqual(
                added_modifed_file_status.working_tree, Status.MODIFIED
            )

    def test_get_status_for_files(self):
        with tempgitdir() as tmpdir:
            (
                _tracked_file,
                _changed_file,
                added_file,
                _staged_changed_file,
                _added_modifed_file,
                _removed_file,
                renamed_file,
                _untracked_file,
            ) = init_test_repo(tmpdir)

            status = get_staged_status((added_file, renamed_file))

            self.assertEqual(len(status), 1)

            added_file_status = status[0]

            self.assertEqual(added_file_status.absolute_path(), added_file)
            self.assertEqual(added_file_status.index, Status.ADDED)
            self.assertEqual(added_file_status.working_tree, Status.UNMODIFIED)


class IsStagedStatusTestCase(unittest.TestCase):
    def test_is_staged_status(self):
        with tempgitdir() as tmpdir:
            init_test_repo(tmpdir)

            status = get_status()

            changed_file_status = status[0]
            renamed_file_status = status[1]
            added_file_status = status[2]
            staged_changed_file_status = status[3]
            added_modifed_file_status = status[4]
            removed_file_status = status[5]

            self.assertFalse(is_staged_status(changed_file_status))
            self.assertTrue(is_staged_status(renamed_file_status))
            self.assertTrue(is_staged_status(added_file_status))
            self.assertTrue(is_staged_status(staged_changed_file_status))
            self.assertTrue(is_staged_status(added_modifed_file_status))
            self.assertFalse(is_staged_status(removed_file_status))


class IsPartiallyStagedStatusTestCase(unittest.TestCase):
    def test_is_partially_staged_status(self):
        with tempgitdir() as tmpdir:
            init_test_repo(tmpdir)

            status = get_status()

            changed_file_status = status[0]
            renamed_file_status = status[1]
            added_file_status = status[2]
            staged_changed_file_status = status[3]
            added_modifed_file_status = status[4]
            removed_file_status = status[5]

            self.assertFalse(is_partially_staged_status(changed_file_status))
            self.assertFalse(is_partially_staged_status(renamed_file_status))
            self.assertFalse(is_partially_staged_status(added_file_status))
            self.assertTrue(
                is_partially_staged_status(staged_changed_file_status)
            )
            self.assertTrue(
                is_partially_staged_status(added_modifed_file_status)
            )
            self.assertFalse(is_partially_staged_status(removed_file_status))



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from typing import Optional

from pontos.helper import deprecated
from pontos.terminal.rich import RichTerminal as Terminal
from pontos.terminal.terminal import Signs
from rich.progress import (
    BarColumn,
    SpinnerColumn,
    TaskProgressColumn,
    TextColumn,
)
from rich.progress import Progress as RichProgress

__all__ = (
    "Terminal",
    "Progress",
    "Signs",
    "bold_info",
    "error",
    "fail",
    "info",
    "ok",
    "out",
    "warning",
)

__term = Terminal()


def ok(message: str) -> None:
    """
    Highlight message as a success/ok in the terminal

    Args:
        message: Message to print
    """
    __term.ok(message)


def fail(message: str) -> None:
    """
    Highlight message as a failure in the terminal

    Args:
        message: Message to print
    """
    __term.fail(message)


def error(message: str) -> None:
    """
    Highlight message as an error in the terminal

    Args:
        message: Message to print
    """
    __term.error(message)


def warning(message: str) -> None:
    """
    Highlight message as a warning in the terminal

    Args:
        message: Message to print
    """
    __term.warning(message)


def info(message: str) -> None:
    """
    Highlight message as an information in the terminal

    Args:
        message: Message to print
    """
    __term.info(message)


def bold_info(message: str) -> None:
    """
    Highlight message as an strong information in the terminal

    Args:
        message: Message to print
    """
    __term.bold_info(message)


def out(message: str):
    """
    Print message to the terminal without highlighting

    Args:
        message: Message to print
    """
    __term.out(message)


@deprecated
def overwrite(
    message: str, new_line: bool = False
):  # pylint: disable=unused-argument
    pass


def _set_terminal(term: Optional[Terminal] = None) -> Terminal:
    global __term  # pylint: disable=global-statement, invalid-name  # noqa: PLW0603
    if not term:
        __term = Terminal()
    else:
        __term = term
    return __term


class Progress(RichProgress):
    def __init__(self, terminal: Terminal) -> None:
        super().__init__(
            SpinnerColumn(),
            TextColumn("[progress.description]{task.description}"),
            BarColumn(),
            TaskProgressColumn(),
            console=terminal._console,
            transient=True,
        )

    def finish_task(self, task_id):
        self.update(task_id, total=1, advance=1)

    def __enter__(  # pylint: disable=useless-super-delegation
        self,
    ) -> "Progress":
        return super().__enter__()  # type: ignore



# SPDX-FileCopyrightText: 2022-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from argparse import Namespace
from typing import Iterable

from autohooks.config import load_config_from_pyproject_toml
from autohooks.precommit.run import autohooks_module_path, check_plugin
from autohooks.settings import AutohooksSettings
from autohooks.terminal import Terminal
from autohooks.utils import get_pyproject_toml_path


def plugins(term: Terminal, args: Namespace) -> None:
    args.plugins_func(term, args)


def print_current_plugins(
    term: Terminal, current_plugins: Iterable[str]
) -> None:
    """
    Print the currently used plugins to the terminal
    """
    term.info("Currently used plugins:")
    with term.indent(), autohooks_module_path():
        if not current_plugins:
            term.print("None")
            return

        for plugin in sorted(current_plugins):
            result = check_plugin(plugin)
            if result:
                term.error(f'"{plugin}": {result}')
            else:
                term.ok(f'"{plugin}"')


# pylint: disable=unused-argument
def list_plugins(term: Terminal, args: Namespace) -> None:
    """
    CLI handler function to list the currently used plugins
    """
    pyproject_toml = get_pyproject_toml_path()
    config = load_config_from_pyproject_toml(pyproject_toml)

    current_plugins = (
        config.settings.pre_commit if config.has_autohooks_config() else []  # type: ignore # pylint:disable = C0301 # noqa:E501
    )
    print_current_plugins(term, current_plugins)


def add_plugins(term: Terminal, args: Namespace) -> None:
    """
    CLI handler function to add new plugins
    """
    pyproject_toml = get_pyproject_toml_path()
    config = load_config_from_pyproject_toml(pyproject_toml)
    plugins_to_add = set(args.name)

    if config.has_autohooks_config():
        settings = config.settings
        existing_plugins = set(settings.pre_commit)  # type: ignore
        all_plugins = plugins_to_add | existing_plugins
        duplicate_plugins = plugins_to_add & existing_plugins
        new_plugins = plugins_to_add - existing_plugins
        settings.pre_commit = all_plugins  # type: ignore

        if duplicate_plugins:
            term.info("Skipped already used plugins:")
            with term.indent():
                for plugin in sorted(duplicate_plugins):
                    term.warning(f'"{plugin}"')
    else:
        all_plugins = plugins_to_add
        new_plugins = plugins_to_add
        settings = AutohooksSettings(pre_commit=all_plugins)
        config.settings = settings

    settings.write(pyproject_toml)  # type: ignore

    if new_plugins:
        term.info("Added plugins:")
        with term.indent():
            for plugin in sorted(new_plugins):
                term.ok(f'"{plugin}"')

    print_current_plugins(term, all_plugins)


def remove_plugins(term: Terminal, args: Namespace) -> None:
    """
    CLI handler function to remove plugins
    """
    pyproject_toml = get_pyproject_toml_path()
    config = load_config_from_pyproject_toml(pyproject_toml)
    plugins_to_remove = set(args.name)

    if config.has_autohooks_config():
        settings = config.settings
        existing_plugins = set(settings.pre_commit)  # type: ignore
        removed_plugins = existing_plugins & plugins_to_remove
        all_plugins = existing_plugins - plugins_to_remove
        skipped_plugins = plugins_to_remove - existing_plugins
        settings.pre_commit = all_plugins  # type: ignore

        if skipped_plugins:
            term.info("Skipped not used plugins:")
            with term.indent():
                for plugin in sorted(skipped_plugins):
                    term.warning(f'"{plugin}"')

        if removed_plugins:
            term.info("Removed plugins:")
            with term.indent():
                for plugin in sorted(removed_plugins):
                    term.ok(f'"{plugin}"')

        settings.write(pyproject_toml)  # type: ignore

        print_current_plugins(term, all_plugins)
    else:
        term.warning("No plugins to remove.")



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

# pylint: disable=invalid-name, protected-access

import os
import unittest
from io import StringIO
from unittest.mock import patch

from autohooks.terminal import Signs, Terminal


class TerminalTestCase(unittest.TestCase):
    def setUp(self):
        os.environ["NO_COLOR"] = "1"
        os.environ["TERM"] = "unknown"
        self.maxDiff = 180
        self.term = Terminal()

    @patch("sys.stdout", new_callable=StringIO)
    def test_error(self, mock_stdout):
        msg = "foo bar"

        expected_msg = f" {Signs.ERROR} {msg}\n"

        self.term.error(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_fail(self, mock_stdout):
        msg = "foo bar baz"

        expected_msg = f" {Signs.FAIL} {msg}\n"

        self.term.fail(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_info(self, mock_stdout):
        msg = "foo bar"

        expected_msg = f" {Signs.INFO} {msg}\n"

        self.term.info(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_bold_info(self, mock_stdout):
        msg = "bold foo bar"

        expected_msg = f" {Signs.INFO} bold foo bar\n"

        self.term.bold_info(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_ok(self, mock_stdout):
        msg = "foo bar"

        expected_msg = f" {Signs.OK} {msg}\n"

        self.term.ok(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_warning(self, mock_stdout):
        msg = "foo bar"

        expected_msg = f" {Signs.WARNING} {msg}\n"

        self.term.warning(msg)

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)

    @patch("sys.stdout", new_callable=StringIO)
    def test_print(self, mock_stdout):
        expected_msg = " foo bar\n"
        self.term.print("foo bar")

        ret = mock_stdout.getvalue()

        self.assertEqual(expected_msg, ret)

    @patch("sys.stdout", new_callable=StringIO)
    def test_with_indent(self, mock_stdout):
        expected_msg = "   foo\n"

        with self.term.indent(2):
            self.term.print("foo")

            ret = mock_stdout.getvalue()

        self.assertEqual(expected_msg, ret)

        # clear the buffer
        mock_stdout.truncate(0)
        mock_stdout.seek(0)

        expected_msg = " bar\n"
        self.term.print("bar")

        ret = mock_stdout.getvalue()

        self.assertEqual(ret, expected_msg)


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Main Plugin API
"""

from autohooks.config import Config
from autohooks.precommit.run import ReportProgress
from autohooks.terminal import bold_info, error, fail, info, ok, out, warning

__all__ = [
    "Config",
    "ReportProgress",
    "error",
    "fail",
    "info",
    "bold_info",
    "ok",
    "out",
    "warning",
]



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from dataclasses import dataclass, field
from enum import Enum
from pathlib import Path
from typing import Iterable, Optional

import tomlkit


class Mode(Enum):
    PIPENV = 1
    PYTHONPATH = 2
    POETRY = 3
    PIPENV_MULTILINE = 4
    POETRY_MULTILINE = 5
    UNDEFINED = -1
    UNKNOWN = -2

    def get_effective_mode(self):
        if self.value == Mode.PIPENV.value:
            return Mode.PIPENV
        if self.value == Mode.PIPENV_MULTILINE.value:
            return Mode.PIPENV_MULTILINE
        if self.value == Mode.POETRY.value:
            return Mode.POETRY
        if self.value == Mode.POETRY_MULTILINE.value:
            return Mode.POETRY_MULTILINE
        return Mode.PYTHONPATH

    @staticmethod
    def from_string(modestring: Optional[str]) -> "Mode":
        if not modestring:
            return Mode.UNDEFINED

        try:
            return Mode[modestring.upper()]
        except KeyError:
            return Mode.UNKNOWN

    def __str__(self) -> str:
        return self.name.lower()  # pylint: disable=no-member


@dataclass
class AutohooksSettings:
    mode: Mode = Mode.UNDEFINED
    pre_commit: Iterable[str] = field(default_factory=list)

    def write(self, filename: Path) -> None:
        """
        Write the current AutohooksSettings to a TOML file

        If the TOML file already exists only the [tool.autohooks] section is
        overridden.
        """
        if filename.exists():
            toml_doc: tomlkit.TOMLDocument = tomlkit.loads(filename.read_text())
        else:
            toml_doc = tomlkit.document()

        if "tool" not in toml_doc:
            toml_doc["tool"] = tomlkit.table(is_super_table=True)
        if "autohooks" not in toml_doc["tool"]:  # type: ignore
            toml_doc["tool"]["autohooks"] = tomlkit.table()  # type: ignore

        config_dict = {
            "mode": str(self.mode.get_effective_mode()),
            "pre-commit": sorted(self.pre_commit),
        }

        toml_doc["tool"]["autohooks"].update(config_dict)  # type: ignore

        filename.write_text(tomlkit.dumps(toml_doc), encoding="utf8")



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

import unittest
from pathlib import Path

from autohooks.api.git import StatusEntry
from autohooks.api.path import is_python_path, match


class MatchTestCase(unittest.TestCase):
    def test_match_list(self):
        patterns = ["*.py", "*.js"]

        self.assertTrue(match(Path("foo.py"), patterns))
        self.assertTrue(match(Path("foo.js"), patterns))
        self.assertTrue(match(Path("path/to/foo.py"), patterns))
        self.assertTrue(match(Path("/root/path/to/foo.py"), patterns))

        self.assertFalse(match(Path("foo.pyc"), patterns))
        self.assertFalse(match(Path("foo.c"), patterns))

    def test_match_tuple(self):
        patterns = ("*.py", "*.js")

        self.assertTrue(match(Path("foo.py"), patterns))
        self.assertTrue(match(Path("foo.js"), patterns))
        self.assertTrue(match(Path("path/to/foo.py"), patterns))
        self.assertTrue(match(Path("/root/path/to/foo.py"), patterns))

        self.assertFalse(match(Path("foo.pyc"), patterns))
        self.assertFalse(match(Path("foo.c"), patterns))

    def test_match_files_in_subdir(self):
        patterns = ("foo/*.py", "bar/*.js")

        self.assertFalse(match(Path("foo.py"), patterns))
        self.assertFalse(match(Path("foo.js"), patterns))
        self.assertFalse(match(Path("foo.pyc"), patterns))
        self.assertFalse(match(Path("foo.c"), patterns))

        self.assertTrue(match(Path("foo/bar.py"), patterns))
        self.assertTrue(match(Path("bar/foo.js"), patterns))

        self.assertFalse(match(Path("foo/bar.js"), patterns))
        self.assertFalse(match(Path("bar/foo.py"), patterns))

    def test_match_status_entry(self):
        patterns = ["*.py", "*.js"]

        self.assertTrue(match(StatusEntry("MM foo.py"), patterns))
        self.assertTrue(match(StatusEntry("MM foo.js"), patterns))
        self.assertTrue(match(StatusEntry("MM path/to/foo.py"), patterns))
        self.assertTrue(match(StatusEntry("MM /root/path/to/foo.py"), patterns))

        self.assertFalse(match(StatusEntry("MM foo.pyc"), patterns))
        self.assertFalse(match(StatusEntry("MM foo.c"), patterns))


class IsPythonPathTestCase(unittest.TestCase):
    def test_is_python_path(self):
        self.assertTrue(is_python_path(Path("foo.py")))
        self.assertTrue(is_python_path(Path("foo.bar.py")))
        self.assertTrue(is_python_path(Path("foo/bar/baz.py")))
        self.assertFalse(is_python_path(Path("foo/bar/baz.pyc")))
        self.assertFalse(is_python_path(Path("foo/bar/baz.py.txt")))
        self.assertFalse(is_python_path(Path("foo/bar/python.txt")))
        self.assertFalse(is_python_path(None))
        self.assertFalse(is_python_path(""))
        self.assertFalse(is_python_path(Path()))


if __name__ == "__main__":
    unittest.main()



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from pathlib import Path
from typing import Any, Dict, List, Optional

import tomlkit

from autohooks.settings import AutohooksSettings, Mode
from autohooks.utils import get_pyproject_toml_path, is_split_env

AUTOHOOKS_SECTION = "tool.autohooks"


class Config:
    """
    Config helper class for easier access to a tree of settings.
    """

    def __init__(self, config_dict: Optional[Dict[str, Any]] = None) -> None:
        """
        Create a new Config from a dictionary.

        Args:
            config_dict: Dictionary to be used for the Config.
        """
        self._config_dict = config_dict or {}

    def get(self, *keys: str) -> "Config":
        """
        Get a sub-config. If a sub-config with the passed keys does not exists
        an empty Config is returned.

        Args:
            *keys: Variable length of keys to resolve.

        Example: ::

            config = Config({"foo": {"bar": {"baz": 1}}})
            baz = config.get("foo", "bar")
            empty_config = config.get("lorem", "ipsum")
        """
        config_dict = self._config_dict

        for key in keys:
            config_dict = config_dict.get(key, {}).copy()

        return Config(config_dict)

    def get_value(self, key: str, default: Any = None) -> Any:
        """
        Get a config value.

        Args:
            key: Key to lookup in the config.
            default: Value to return if key is not in the config. By default
                None is returned.

        Example: ::

            config = Config({"foo": {"bar": {"baz": 1}}})
            value = config.get("foo", "bar").get_value("baz")
        """
        return self._config_dict.get(key, default)

    def is_empty(self) -> bool:
        """
        Returns True if the config has no data.
        """
        return not bool(self._config_dict)

    def has_key(self, key: str) -> bool:
        """
        Returns True if the key is in the config.
        """
        return key in self._config_dict


def _gather_mode(mode_string: Optional[str]) -> Mode:
    """
    Gather the mode from a mode string
    """
    mode = Mode.from_string(mode_string)
    is_virtual_env = mode == Mode.PIPENV or mode == Mode.POETRY
    if is_virtual_env and not is_split_env():
        if mode == Mode.POETRY:
            mode = Mode.POETRY_MULTILINE
        else:
            mode = Mode.PIPENV_MULTILINE
    return mode


class AutohooksConfig:
    def __init__(
        self,
        *,
        settings: Optional[AutohooksSettings] = None,
        config: Optional[Config] = None,
    ) -> None:
        self.config = Config() if config is None else config
        self.settings = settings

    def get_config(self) -> Config:
        return self.config

    def has_autohooks_config(self) -> bool:
        return self.settings is not None

    def get_pre_commit_script_names(self) -> List[str]:
        return self.settings.pre_commit if self.has_autohooks_config() else []  # type: ignore # pylint:disable # noqa: E501

    def get_mode(self) -> Mode:
        return (
            self.settings.mode  # type: ignore
            if self.has_autohooks_config()
            else Mode.UNDEFINED
        )

    @staticmethod
    def from_dict(config_dict: Dict[str, Any]) -> "AutohooksConfig":
        """
        Create a new AutohooksConfig from a dictionary

        Args:
            config_data: A dictionary containing the config data

        Returns:
            A new AutohooksConfig
        """
        config = Config(config_dict)
        autohooks_dict = config.get("tool", "autohooks")
        if autohooks_dict.is_empty():
            settings = None
        else:
            settings = AutohooksSettings(
                mode=_gather_mode(autohooks_dict.get_value("mode")),
                pre_commit=autohooks_dict.get_value("pre-commit", []),
            )
        return AutohooksConfig(settings=settings, config=config)

    @staticmethod
    def from_string(content: str) -> "AutohooksConfig":
        """
        Load an AutohooksConfig from a string

        Args:
            content: The content of the config

        Returns:
            A new AutohooksConfig
        """
        config_dict = tomlkit.loads(content)
        return AutohooksConfig.from_dict(config_dict)

    @staticmethod
    def from_toml(toml_file: Path) -> "AutohooksConfig":
        """
        Load an AutohooksConfig from a TOML file

        Args:
            toml_file: Path for the toml file to load

        Returns:
            A new AutohooksConfig
        """
        return AutohooksConfig.from_string(toml_file.read_text())


def load_config_from_pyproject_toml(
    pyproject_toml: Optional[Path] = None,
) -> AutohooksConfig:
    """
    Load an AutohooksConfig from a pyproject.toml file

    If no path to the pyproject.toml file is passed the path will be determined
    from the current working directory and the project.

    Args:
        pyproject_toml: Path to the pyproject.toml file.

    Returns:
        A new AutohooksConfig
    """
    if pyproject_toml is None:
        pyproject_toml = get_pyproject_toml_path()

    if not pyproject_toml.exists():
        return AutohooksConfig()

    return AutohooksConfig.from_toml(pyproject_toml)



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#
"""
Plugin API for path checking
"""

import fnmatch
import os
from os import PathLike
from pathlib import Path
from typing import Iterable, Optional


def is_python_path(path: Optional[Path]) -> bool:
    """
    Function to check if path is a Python file.

    Args:
        path: A path to a file.

    Returns:
        True if path is a Python file.
    """
    if not path:
        return False
    return path.match("*.py")


def match(path: PathLike, pattern_list: Iterable[str]) -> bool:
    """
    Check if a path like object matches to one of the patterns.

    Internally fnmatch is used.
    See https://docs.python.org/3/library/fnmatch.html for details.

    Arguments:
        path: :py:class:`os.PathLike` to check if it matches to one of the
            patterns.
        pattern_list: Iterable (e.g tuple or list) of patterns to
            match against the path..

    Returns:
        True if path matches a pattern of the list.
    """
    for pattern in pattern_list:
        if fnmatch.fnmatch(os.fspath(path), pattern):
            return True
    return False



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from argparse import Namespace

from autohooks.config import (
    get_pyproject_toml_path,
    load_config_from_pyproject_toml,
)
from autohooks.hooks import PreCommitHook
from autohooks.settings import AutohooksSettings, Mode
from autohooks.terminal import Terminal


def install_hooks(term: Terminal, args: Namespace) -> None:
    pre_commit_hook = PreCommitHook()
    pyproject_toml = get_pyproject_toml_path()
    config = load_config_from_pyproject_toml(pyproject_toml)

    if pre_commit_hook.exists() and not args.force:
        term.warning(
            "autohooks pre-commit hook is already"
            f" installed at {pre_commit_hook}."
        )
        with term.indent():
            term.print()
            term.info(
                "Run 'autohooks activate --force' to override the current "
                "installed pre-commit hook."
            )
            term.info(
                "Run 'autohooks check' to validate the current status of "
                "the installed pre-commit hook."
            )
    else:
        if args.mode:
            mode = Mode.from_string(args.mode)
        else:
            mode = config.get_mode().get_effective_mode()

        if not config.has_autohooks_config():
            settings = AutohooksSettings(mode=mode)
            config.settings = settings
            settings.write(pyproject_toml)

            term.ok(f"autohooks settings written to {pyproject_toml}.")
        elif args.force:
            settings = config.settings  # type: ignore
            settings.mode = mode
            settings.write(pyproject_toml)

            term.ok(f"autohooks settings written to {pyproject_toml}.")

        pre_commit_hook.write(mode=mode)

        term.ok(
            f"autohooks pre-commit hook installed at {pre_commit_hook}"
            f" using {mode} mode."
        )



# SPDX-FileCopyrightText: 2022-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from autohooks.api.git import StatusEntry, get_diff
from tests import tempgitdir
from tests.api.git import GitTestCase, git_add, git_commit


class DiffTestCase(GitTestCase):
    def test_get_diff_from_status(self):
        with tempgitdir() as tmpdir:
            test_file = tmpdir / "foo.txt"
            test_file.write_text(
                "Lorem\nipsum\ndolor\nsit\namet", encoding="utf8"
            )

            git_add(test_file)
            git_commit()

            test_file.write_text("ipsum\ndolor\nsit\namet", encoding="utf8")
            status = StatusEntry("M  foo.txt", tmpdir)
            diff = get_diff((status,))

            expected_diff = """--- a/foo.txt
+++ b/foo.txt
@@ -1,4 +1,3 @@
-Lorem
 ipsum
 dolor
 sit"""
            self.assertIn(expected_diff, diff)

    def test_get_diff(self):
        with tempgitdir() as tmpdir:
            test_file = tmpdir / "foo.txt"
            test_file.write_text(
                "Lorem\nipsum\ndolor\nsit\namet", encoding="utf8"
            )

            git_add(test_file)
            git_commit()

            test_file.write_text("ipsum\ndolor\nsit\namet", encoding="utf8")
            diff = get_diff()

            expected_diff = """--- a/foo.txt
+++ b/foo.txt
@@ -1,4 +1,3 @@
-Lorem
 ipsum
 dolor
 sit"""
            self.assertIn(expected_diff, diff)



# SPDX-FileCopyrightText: 2019-2024 Greenbone AG
#
# SPDX-License-Identifier: GPL-3.0-or-later
#

from autohooks.__version__ import __version__ as _current_version


def get_version() -> str:
    """
    Returns the current version of autohooks
    """
    return _current_version
